/*----------------------------------------------------------------------------
 *      RL-ARM - RTX
 *----------------------------------------------------------------------------
 *      Name:    BLINKY.C
 *      Purpose: RTX example program
 *----------------------------------------------------------------------------
 *      This code is part of the RealView Run-Time Library.
 *      Copyright (c) 2006-2013 KEIL - An ARM Company. All rights reserved.
 *---------------------------------------------------------------------------*/
#include "cmsis_os.h"                   // ARM::CMSIS:RTOS:Keil RTX
#include <MKM34Z5.h>
#include "lcd.h"
#include "adc.h"
#include "gpio.h"


osThreadId tid_phaseA;                  /* Thread id of thread: phase_a      */
osThreadId tid_phaseB;                  /* Thread id of thread: phase_b      */
osThreadId tid_phaseC;                  /* Thread id of thread: phase_c      */
osThreadId tid_phaseD;                  /* Thread id of thread: phase_d      */
osThreadId tid_clock;                   /* Thread id of thread: clock        */
osThreadId tid_lcd;                     /* Thread id of thread: lcd          */


/* LEDs definition                                                            */
#define LED_G  PIN5        /* PTE5 */ 
#define LED_R  PIN1        /* PTF1 */ 
#define LED_Y  PIN1        /* PTD1 */ 
#define LED_O  PIN1        /* PTC1 */ 
#define LED_NUM     4                   /* Number of user LEDs                */

static const char digits[]=
{
  '0','1','2','3','4','5','6','7','8','9','A','b','c','d','E','F','8'
};

const uint32_t led_mask[] = {1UL << 5, 1UL << 1, 1UL << 1, 1UL << 1};
const uint32_t gpio_id[] = {GPIOE_BASE, GPIOF_BASE, GPIOD_BASE, GPIOC_BASE};

#define LED_A      1
#define LED_B      2
#define LED_C      3
#define LED_D      1
#define LED_CLK    0


/*----------------------------------------------------------------------------
  Function that initializes LEDs
 *----------------------------------------------------------------------------*/
void LED_init(void) {
	/*Enable Port E/F/D/C Clock */
  SIM->SCGC5 |= (1 << SIM_SCGC5_PORTE_SHIFT) | (1 << SIM_SCGC5_PORTF_SHIFT) |\
	              (1 << SIM_SCGC5_PORTD_SHIFT) | (1 << SIM_SCGC5_PORTC_SHIFT);

  /*Port Initialization */
  PORT_Init (PORTE, PORT_MODULE_LED_MODE, LED_G);
  PORT_Init (PORTF, PORT_MODULE_LED_MODE, LED_R);
  PORT_Init (PORTD, PORT_MODULE_LED_MODE, LED_O);
  PORT_Init (PORTC, PORT_MODULE_LED_MODE, LED_Y);
  /*GPIO Initialization */
  GPIO_Init (GPIOE, GPIO_OUT_MODE, LED_G);
  GPIO_Init (GPIOF, GPIO_OUT_MODE, LED_R);
  GPIO_Init (GPIOD, GPIO_OUT_MODE, LED_O);
  GPIO_Init (GPIOC, GPIO_OUT_MODE, LED_Y);
  /*All LEDs Off */
  GPIO_Set  (GPIOC, LED_Y);
  GPIO_Set  (GPIOE, LED_G);
  GPIO_Set  (GPIOF, LED_R);
  GPIO_Set  (GPIOD, LED_O);
}

/*----------------------------------------------------------------------------
  Function that turns on requested LED
 *----------------------------------------------------------------------------*/
void LED_On (unsigned int num) {
  ((GPIO_Type *)gpio_id[num])->PDOR &= ~led_mask[num];
}

/*----------------------------------------------------------------------------
  Function that turns off requested LED
 *----------------------------------------------------------------------------*/
void LED_Off (unsigned int num) {
  ((GPIO_Type *)gpio_id[num])->PDOR |= led_mask[num];
}

/*----------------------------------------------------------------------------
 *      Function 'signal_func' called from multiple threads
 *---------------------------------------------------------------------------*/
void signal_func (osThreadId tid)  {
  osSignalSet(tid_clock, 0x0100);           /* set signal to clock thread    */
  osDelay(500);                             /* delay 500ms                   */
  osSignalSet(tid_clock, 0x0100);           /* set signal to clock thread    */
  osDelay(500);                             /* delay 500ms                   */
  osSignalSet(tid, 0x0001);                 /* set signal to thread 'thread' */
  osDelay(500);                             /* delay 500ms                   */
}

/*----------------------------------------------------------------------------
 *      Thread 1 'phaseA': Phase A output
 *---------------------------------------------------------------------------*/
void phaseA (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    LED_On (LED_A);
    signal_func(tid_phaseB);                /* call common signal function   */
    LED_Off(LED_A);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 2 'phaseB': Phase B output
 *---------------------------------------------------------------------------*/
void phaseB (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    LED_On (LED_B);
    signal_func(tid_phaseC);                /* call common signal function   */
    LED_Off(LED_B);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 3 'phaseC': Phase C output
 *---------------------------------------------------------------------------*/
void phaseC (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    LED_On (LED_A);
    signal_func(tid_phaseD);                /* call common signal function   */
    LED_Off(LED_A);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 4 'phaseD': Phase D output
 *---------------------------------------------------------------------------*/
void phaseD (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    LED_On (LED_D);
    signal_func(tid_phaseA);                /* call common signal function   */
    LED_Off(LED_D);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 5 'clock': Signal Clock
 *---------------------------------------------------------------------------*/
void clock (void  const *argument) {
  for (;;) {
    osSignalWait(0x0100, osWaitForever);    /* wait for an event flag 0x0100 */
    LED_On (LED_CLK);
    osDelay(80);                            /* delay 80ms                    */
    LED_Off(LED_CLK);
  }
}



/*---------------------------------------------------------------------------------------
 *      Thread 'lcd': LCD Control task
 *--------------------------------------------------------------------------------------*/
void lcd (void  const *argument) {
  uint32_t tmp32;
  uint16_t tmp16;
  uint8_t  tmpNum;

  _LCD_CHAR_V_S3_ON;
  _LCD_CHAR_V_S6_ON;
  _LCD_CHAR_V_S7_ON;
  _LCD_CHAR_V_S9_ON;
  _LCD_CHAR_V_S11_ON;
  _LCD_CHAR_V_S12_ON;
  _LCD_CHAR_V_S13_ON;
  _LCD_DOT_P3_ON;

  for (;;) {
    ADC_Start(CHA,AD10);
    while (!ADC_Ready (CHA));
    tmp16 = ADC_Read(CHA);
    tmp32 = 50050 * tmp16;
  
    tmpNum = (uint8) (tmp32/1000000000);
    tmp32 = tmp32 % 1000000000;
    LCD_Delete_Char(3);
    LCD_Write_Char ( digits[tmpNum] );

    tmpNum = (uint8) (tmp32/100000000);
    tmp32 = tmp32 % 100000000;
    LCD_Delete_Char(2);
    LCD_Write_Char ( digits[tmpNum] );

    tmpNum = (uint8) (tmp32/10000000);
    tmp32 = tmp32 % 10000000;
    LCD_Delete_Char(1);
    LCD_Write_Char ( digits[tmpNum] );
  
    tmpNum = (uint8) (tmp32/1000000);
    LCD_Delete_Char(0);
    LCD_Write_Char ( digits[tmpNum] );

    osDelay(200);                            /* delay 200ms                    */
  }
}

osThreadDef(phaseA, osPriorityNormal, 1, 0);
osThreadDef(phaseB, osPriorityNormal, 1, 0);
osThreadDef(phaseC, osPriorityNormal, 1, 0);
osThreadDef(phaseD, osPriorityNormal, 1, 0);
osThreadDef(clock,  osPriorityNormal, 1, 0);
osThreadDef(lcd,    osPriorityNormal, 1, 0);

/*----------------------------------------------------------------------------
 *        Main: Initialize and start RTX Kernel
 *---------------------------------------------------------------------------*/
int main (void) {

  /* Disable NMI */
  SIM->CTRL_REG |= (1 << SIM_CTRL_REG_NMIDIS_SHIFT);
  
  LED_init ();                              /* Initialize the LEDs           */

  /* Enable ADC Clock */
  SIM->SCGC6 |= (1 << SIM_SCGC6_ADC_SHIFT);
  ADC_Init (ADC_MODULE_16B_SWTRG_XREF_CONFIG, HWAVG_32, ADC_CH_SE_POLL_CONFIG(AD_OFF),
            ADC_CH_DISABLE_CONFIG, ADC_CH_DISABLE_CONFIG, ADC_CH_DISABLE_CONFIG, 
            0, NULL);
  /* Enable SLCD Clock */
  SIM->SCGC5 |= (1 << SIM_SCGC5_SLCD_SHIFT);
  LCD_Init         (LCD_CHARGEPUMP_EN_SOURCE_ALT_BACKPLANES_8,                 \
                    LCD_FRONT_PLANE_PINS_TWR, LCD_BACK_PLANE_PINS_TWR,         \
                    LCD_ASSIGN_BACK_PLANES_TWR);
  
 
  tid_phaseA = osThreadCreate(osThread(phaseA), NULL);
  tid_phaseB = osThreadCreate(osThread(phaseB), NULL);
  tid_phaseC = osThreadCreate(osThread(phaseC), NULL);
  tid_phaseD = osThreadCreate(osThread(phaseD), NULL);
  tid_clock  = osThreadCreate(osThread(clock),  NULL);
  tid_lcd    = osThreadCreate(osThread(lcd),    NULL);

  osSignalSet(tid_phaseA, 0x0001);          /* set signal to phaseA thread   */

  osDelay(osWaitForever);
  while(1);
}
